<?php

namespace Drp\NativeEventApp\Auth;

use Drp\NativeEventApp\Contracts\Obfuscatable;
use Drp\NativeEventApp\Enums\AuthMethod;
use Illuminate\Support\Str;

class ContactDetail
{
    /** @var AuthMethod */
    private $method; //phpcs:ignore

    /** @var mixed */
    private $value; //phpcs:ignore

    /**
     * @param mixed $value
     */
    public function __construct(AuthMethod $method, ?string $value)
    {
        $this->method = $method;
        $this->value = $method->toObject($value);
    }

    public function __isset(string $key): bool
    {
        return (bool) $this->{$key};
    }

    /**
     * @return mixed
     */
    public function __get(string $key)
    {
        $methodName = 'get' . Str::camel($key);
        if (method_exists($this, $methodName)) {
            return $this->$methodName();
        }

        return $this->{$key};
    }

    /**
     * @return mixed
     */
    public function __call(string $name, array $arguments)
    {
        return $this->value->$name(...$arguments);
    }

    public function __toString(): string
    {
        return (string) $this->value;
    }

    public function getPreview(): string
    {
        if ($this->value instanceof Obfuscatable) {
            return $this->value->obfuscated();
        }

        return (string) $this->value;
    }
}
