<?php

namespace Drp\NativeEventApp\Forms;

use Drp\NativeEventApp\Exceptions\FormNotFound;
use Drp\NativeEventApp\Models\Registration;
use Illuminate\Support\Arr;
use Illuminate\Validation\ValidationException;

class FormRegistry
{
    private $callbacks = [];

    public function process(string $formName, string $data, Registration $registration): array
    {
        app()->instance("eventpal-form.data", json_decode($data, true));

        try {
            return [
                'status' => 200,
                'message' => '',
                'submissionData' => app()->call($this->getCallback($formName), [
                    'registration' => $registration,
                ])->toJson(),
            ];
        } catch (ValidationException $exception) {
            throw new \Nuwave\Lighthouse\Exceptions\ValidationException($exception->validator);
        }
    }

    /**
     * @param callable|string $callback
     */
    public function handle(string $formName, $callback): void
    {
        $this->callbacks[$formName] = $callback;
    }

    /**
     * @return array|string|callable
     */
    protected function getCallback(string $formName)
    {
        // phpcs:ignore
        // TODO: add in more exceptions and handling etc for this

        $callback = Arr::get($this->callbacks, $formName, null);

        throw_if($callback === null, FormNotFound::byKey($formName));

        if (is_string($callback) && class_exists($callback)) {
            $callback = [
                app($callback),
                '__invoke',
            ];
        }

        return $callback;
    }
}
