<?php

namespace Drp\NativeEventApp\Test\Api;

use Drp\NativeEventApp\FileUploads\Facades\FileUpload as FileUploadFacade;
use Drp\NativeEventApp\Test\Support\InteractsWithFileUploads;
use Drp\NativeEventApp\Test\TestCase;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class RetrieveFileUploadTest extends TestCase
{
    use InteractsWithFileUploads;

    /** @test */
    public function it_can_retrieve_a_valid_file_response(): void
    {
        $this->withoutExceptionHandling();
        Storage::fake(config('native-event-app.files.local.disk'));
        $file = UploadedFile::fake()->image('test.jpg');

        $uuid = (string) Str::uuid();
        $url = $this->createSignedUrl($uuid);

        $response = $this->call('PUT', $url, [], [], [], [], $file->get());

        $response->assertStatus(200);
        $response->assertJson([
            'id' => $uuid,
        ]);

        // Fake form submission
        FileUploadFacade::storeReference(
            $uuid,
            1
        );

        $response = $this->get(route('eventpal.sdk.file-upload.retrieve', $uuid));

        $response->assertStatus(200);
        $response->assertHeader('Content-Type', 'image/jpeg');
    }

    /** @test */
    public function it_returns_a_404_not_found_if_uuid_doesnt_exist(): void
    {
        $this->get(route('eventpal.sdk.file-upload.retrieve', (string) Str::uuid()), [])
            ->assertStatus(404);
    }
}
