<?php

namespace Drp\NativeEventApp\Test\Unit\FileUploads;

use Carbon\Carbon;
use Drp\NativeEventApp\FileUploads\Facades\FileUpload as FileUploadFacade;
use Drp\NativeEventApp\FileUploads\S3Driver;
use Drp\NativeEventApp\Models\FileUpload;
use Drp\NativeEventApp\Test\TestCase;
use Illuminate\Support\Str;

class S3DriverTest extends TestCase
{
    use FileUploadContractTest;

    protected function setUp(): void
    {
        $this->markTestSkipped('Live Amazon S3 Test: Remove setUp and add credentials to test');
    }

    public function driver(): S3Driver
    {
        config()->set('native-event-app.files.s3.key', 'ENTER_ACCESS_KEY_ID');
        config()->set('native-event-app.files.s3.secret', 'ENTER_ACCESS_SECRET');
        config()->set('native-event-app.files.s3.bucket', 'ENTER_BUCKET_NAME');

        return FileUploadFacade::createS3Driver();
    }

    public function storeFile(string $path, string $uuid): void
    {
        $url = $this->driver()->createUrl($uuid, '+1 minute')['url'];

        $ch = curl_init();
        $options = [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'PUT',
            CURLOPT_POSTFIELDS => file_get_contents($path),
            CURLOPT_HTTPHEADER => [
                'Content-Type: image/png',
            ],
        ];
        curl_setopt_array($ch, $options);
        curl_exec($ch);
        curl_close($ch);
    }

    public function assertFileCorrectlyUploaded(FileUpload $fileUpload, string $uuid): void
    {
        $this->assertEquals(config('native-event-app.files.s3.bucket'), $fileUpload->disk);
        $this->assertEquals($uuid, $fileUpload->path);

        $this->assertTrue($this->driver()->getStorageClient()->doesObjectExist(
            config('native-event-app.files.s3.bucket'),
            $uuid
        ));
    }

    public function assertFileUrlCorrectlyGenerated(FileUpload $fileUpload, string $fileUrl): void
    {
        $this->assertStringContainsString($fileUpload->disk, $fileUrl);
        $this->assertStringContainsString(
            config('native-event-app.files.s3.region') . '.amazonaws.com',
            $fileUrl
        );
        $this->assertStringContainsString($fileUpload->uuid, $fileUrl);
    }

    /** @test */
    public function it_stores_the_generated_s3_url_in_the_cache_with_an_expiry(): void
    {
        Carbon::setTestNow('14-05-2020 13:00');

        $driver = $this->driver();

        $uuid = (string) Str::uuid();
        $cacheKey = 'S3_IMAGES.' . $uuid;
        $this->assertEquals($cacheKey, $this->driver()->getCacheKey($uuid));

        $this->storeFile(__DIR__ . '/test.png', $uuid);
        $fileUpload = $driver->storeReference($uuid, 1);

        $url = $driver->getFileUrl($fileUpload);

        $this->assertTrue(cache()->has($cacheKey));
        $this->assertEquals($url, cache()->get($cacheKey));

        // Cache expiry is set to 20 minutes in driver.
        Carbon::setTestNow('14-05-2020 13:30');

        $this->assertFalse(cache()->has($cacheKey));
    }
}
