<?php

namespace Drp\NativeEventApp\Unit\Modules;

use Drp\NativeEventApp\Facades\ModuleRegistry;
use Drp\NativeEventApp\Test\TestCase;
use Drp\NativeEventApp\Test\Support\InteractsWithModules;
use Nuwave\Lighthouse\Schema\TypeRegistry;


class ModuleRegistryTest extends TestCase
{
    use InteractsWithModules;

    /** @test */
    public function returns_modules_as_enum_data()
    {
        $this->assertContainsFakeModule(ModuleRegistry::modulesForEnums());
    }

    /** @test */
    public function registers_modules_as_graphQL_enum()
    {
        /** @var \Drp\NativeEventApp\Enums\ModuleType $enum */
        $moduleTypes = app(TypeRegistry::class)->get('ModuleType');

        $values = array_map(function ($enum) {
            return [
                'name' => $enum->name,
                'value' => $enum->value,
                'description' => $enum->description,
            ];
        }, $moduleTypes->getValues());

        $this->assertContainsFakeModule($values);
    }

    private function assertContainsFakeModule(array $value): void
    {
        $this->assertContains(
            [
                'name' => 'fake_module',
                'value' => 'fake_module',
                'description' => 'Fake module',
            ],
            $value
        );
    }
}